# -*- coding: utf-8 -*-
# 何も問題なさげにみえて、そうでもない。
# distutils の制限で、Geographiclib-1.40 がこの階層よりも上、
# つまり「../」を含んでいると、distutils はこれを制御しようと
# しない。(output_dir="_build" は意味をなくす。)
# -----------------------------------
import os
import re
from distutils import log
from distutils import ccompiler, sysconfig, dep_util, errors

log.set_verbosity(1)  # INFO

cc = ccompiler.new_compiler()
sysconfig.customize_compiler(cc)
cc.set_include_dirs(['Geographiclib-1.40/include'])
if cc.compiler_type == 'msvc':
    extra_preargs = ['-EHsc']

# build geographiclib dynamic link library
import glob

srcs = list(glob.glob("Geographiclib-1.40/src/*.cpp"))
objs = ["_build/" + re.sub(r"\.cpp$", cc.obj_extension, src) for src in srcs]

# compile if each source is newer than its corresponding target.
for src in dep_util.newer_pairwise(srcs, objs)[0]:
    # Windows 向け「__declspec(dllexport)」の対応方法はプロジェクトによりマチマチだが、
    # GeographicLib と同じやり方が最も標準的。(MS 推奨でもある。)
    # GeographicLib についてのそれについては、Constants.hpp 参照。
    cc.compile([src],
               "_build",
               extra_preargs=extra_preargs + [
            "-DGEOGRAPHICLIB_SHARED_LIB=1", "-DGeographicLib_EXPORTS"])
cc.link_shared_lib(objs, "GeographicLib", ".")
#
log.set_verbosity(0)  # 

for fn in glob.glob("GeographicLib-1.40/examples/*.cpp"):
    objects = cc.compile([fn], extra_preargs=extra_preargs)
    execname = os.path.splitext(os.path.basename(fn))[0]
    try:
        cc.link_executable(objects, execname, libraries=['GeographicLib'])
    except errors.LinkError as e:
        # static link 版では起こらないリンクエラーが出るのがいる。
        # 「distutils で 非python な C/C++ ビルド」のお題の範疇外
        # なので気にしていないが、まぁこれは良く起こる。
        # おそらくコンパイラオプションかリンカオプションに何か追加
        # すれば取れると思う。(典型的には /MD とかだな。)
        print(e)
